/**
 * AuslagenTatbestand.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlAttribute;

/**
 * Von GebuehrenTatbestand abgeleitete Klasse, für Auslagen und
 * streitwertunabhängige Gebühren.
 * 
 * @author Peter Schuster (setrok)
 * @date 06.05.2014
 * 
 */
class AuslagenTatbestand extends GebuehrenTatbestand {

	private static final long serialVersionUID = -7242122091272139243L;
	private double betrag;

	/**
	 * Dieser Konstruktor (ohne die Höhe der Auslagen) kann benutzt werden, wenn
	 * die Höhe variabel, also nicht immer dieselbe ist (etwa bei Kosten der
	 * Beweisaufnahme). Eine Abfrage der Auslagen liefert so lange -1.0, bis ein
	 * anderer Wert mit {@link #setBetrag(double)} festgesetzt wird.
	 * 
	 * @param bezeichnung
	 *            Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *            Verzeichnis als String, z.B. "Nr. 1100 KV"
	 * @param gebuehrenKlasse
	 *            Klasse der Gebühr, also den Verweis auf die Klasse eines
	 *            Ablegers von GebuehrenTabelle, also entweder
	 *            GerichtsGebuehrenTabelle.class oder
	 *            AnwaltsGebuehrenTabelle.class
	 * 
	 * @see eu.gronos.GebuehrenSatzTatbestand.GebuehrenTatbestand#Constructor(String,double,Class)
	 */
	public AuslagenTatbestand(String bezeichnung,
			Class<? extends GebuehrenTabelle> gebuehrenKlasse) {
		this(bezeichnung, -1.0, gebuehrenKlasse);
	}

	/**
	 * Dieser Konstruktor (mit Höhe der Auslagen) kann benutzt werden, wenn die
	 * Höhe der Auslage bei Initialisieren bereits feststeht, z.B. weil es
	 * eigentlich eine Gebühr mit fester (streitwertunabhängiger) Höhe ist, z.B.
	 * die Aktenversandpauschale.
	 * 
	 * @param bezeichnung
	 *            Die Bezeichnung des Gebührentatbestands nach dem jeweiligen
	 *            Verzeichnis als String, z.B. "Nr. 1100 KV"
	 * @param auslagen
	 *            die Höhe der Auslagen oder der streitwertunabhängigen
	 *            Gebühr/Pauschale, oder ein negativer Wert (-1.0), wenn diese
	 *            noch nicht feststeht und später gesetzt werden soll.
	 * @param gebuehrenKlasse
	 *            Klasse der Gebühr, also den Verweis auf die Klasse eines
	 *            Ablegers von GebuehrenTabelle, also entweder
	 *            GerichtsGebuehrenTabelle.class oder
	 *            AnwaltsGebuehrenTabelle.class
	 */
	public AuslagenTatbestand(String bezeichnung, double auslagen,
			Class<? extends GebuehrenTabelle> gebuehrenKlasse) {
		// super(bezeichnung, -1.0, gebuehrenKlasse);
		super(bezeichnung, gebuehrenKlasse);
		this.betrag = auslagen;
	}

	/**
	 * Konstruktor ohne Parameter für {@link JAXB}
	 * 
	 */
	public AuslagenTatbestand() {
		super();
	}

	/**
	 * @return gibt das Feld {@link #betrag} als double zurück.
	 */
	@XmlAttribute(name="betrag")
	public double getBetrag() {
		return betrag;
	}

	/**
	 * @param betrag
	 *            d. {@link #betrag}, d. gesetzt werden soll als double
	 */
	public void setBetrag(double betrag) {
		this.betrag = betrag;
	}

	/**
	 * Die toString() Methode der Klasse AuslagenTatbestand liefert einen String
	 * mit der Bezeichnung des Gebührentatbestands und der Höhe der Auslagen in
	 * Klammern.
	 * 
	 * @return einen String nach dem Format "Text (Auslagen)", z.B.
	 *         "Verfahrensgebühr, Ziffer xxxx VVRVG (2.000,00 EUR)"
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		if (getBetrag() < 0)
			return String.format("%s (Höhe noch nicht festgelegt)",
					getBezeichnung());
		else
			return String.format("%s (%,.2f EUR)", getBezeichnung(),
					getBetrag());
	}
}
