/**
 * GebuehrensatzRenderer.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;

/**
 * Stellt Gebührensätze dar mit einer Nachkommastelle dar.
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date: 29.07.2014
 *
 */
class GebuehrenRenderer extends JLabel implements ListCellRenderer<Double>,
		TableCellRenderer {

	private static final long serialVersionUID = -204281833673733183L;

	/**
	 * Rendert Tabellenzellen als JLabel.
	 * 
	 * @param table
	 *            the JTable that is asking the renderer to draw; can be null
	 * @param value
	 *            the value of the cell to be rendered. It is up to the specific
	 *            renderer to interpret and draw the value. For example, if
	 *            value is the string "true", it could be rendered as a string
	 *            or it could be rendered as a check box that is checked. null
	 *            is a valid value
	 * @param isSelected
	 *            true if the cell is to be rendered with the selection
	 *            highlighted; otherwise false
	 * @param hasFocus
	 *            if true, render cell appropriately. For example, put a special
	 *            border on the cell, if the cell can be edited, render in the
	 *            color used to indicate editing
	 * @param row
	 *            the row index of the cell being drawn. When drawing the
	 *            header, the value of row is -1
	 * @param column
	 *            the column index of the cell being drawn
	 * @return ein JLabel, das entweder einen Euro-Betrag mit zwei
	 *         Nachkommastellen (bei einem Auslagentatbestand) oder einen
	 *         Gebührensatz mit einer Nachkommastelle (ansonsten) darstellt.
	 * 
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable,
	 *      java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		if (table.getModel() instanceof GebuehrenTatbestandTableModel) {
			GebuehrenTatbestandTableModel gttm = ((GebuehrenTatbestandTableModel) table
					.getModel());
			GebuehrenTatbestand tb = gttm.getAllValues().get(row);
			if (tb instanceof AuslagenTatbestand)
				setText(String.format("%,.2f EUR", value));
			else if (tb instanceof GebuehrenSatzTatbestand)
				setText(String.format("%,.1f-fach", value));
			else if (tb instanceof MehrwertsteuerTatbestand)
				setText(String.format("%,.1f%%", (Double) value * 100.0));
			setHorizontalAlignment(SwingConstants.RIGHT);
		} else
			setText(Double.toString((Double) value));
		if (isSelected) {
			/*
			 * Wenn die Zeile ausgewählt ist, müssen die Farben entsprechend
			 * gesetzt werden.
			 */
			setBackground(table.getSelectionBackground());
			setForeground(table.getSelectionForeground());
			setOpaque(true);
		} else {/* sonst zurücksetzen */
			setBackground(table.getBackground());
			setForeground(table.getForeground());
		}
		return this;
	}

	/**
	 * getListCellRendererComponent:
	 * 
	 * @param list
	 *            The JList we're painting.
	 * @param value
	 *            the value of the cell to be rendered. It is up to the specific
	 *            renderer to interpret and draw the value. For example, if
	 *            value is the string "true", it could be rendered as a string
	 *            or it could be rendered as a check box that is checked. null
	 *            is a valid value
	 * @param index
	 *            The cells index.
	 * @param isSelected
	 *            True if the specified cell was selected.
	 * @param cellHasFocus
	 *            True if the specified cell has the focus.
	 * @return ein JLabel, das entweder einen Euro-Betrag mit zwei
	 *         Nachkommastellen (bei einem Auslagentatbestand) oder einen
	 *         Gebührensatz mit einer Nachkommastelle (ansonsten) darstellt.
	 * 
	 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
	 *      java.lang.Object, int, boolean, boolean)
	 */
	@Override
	public Component getListCellRendererComponent(JList<? extends Double> list,
			Double value, int index, boolean isSelected, boolean cellHasFocus) {
		setText(String.format("%,.1f", value));
		if (isSelected) {
			/*
			 * Wenn die Zeile ausgewählt ist, müssen die Farben entsprechend
			 * gesetzt werden.
			 */
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
			setOpaque(true);
		} else {/* sonst nicht */
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		return this;
	}

}
